Documentation
¶
Overview ¶
Implements the echo Logger proxy to log/slog.
Index ¶
- func SetDefault(p *Proxy)
- type Logger
- type Proxy
- func (p *Proxy) Debug(args ...interface{})
- func (p *Proxy) Debugf(format string, args ...interface{})
- func (p *Proxy) Debugj(j log.JSON)
- func (p *Proxy) Error(args ...interface{})
- func (p *Proxy) Errorf(format string, args ...interface{})
- func (p *Proxy) Errorj(j log.JSON)
- func (p *Proxy) Fatal(args ...interface{})
- func (p *Proxy) Fatalf(format string, args ...interface{})
- func (p *Proxy) Fatalj(j log.JSON)
- func (p *Proxy) Info(args ...interface{})
- func (p *Proxy) Infof(format string, args ...interface{})
- func (p *Proxy) Infoj(j log.JSON)
- func (p *Proxy) Level() log.Lvl
- func (p *Proxy) Output() io.Writer
- func (p *Proxy) Panic(args ...interface{})
- func (p *Proxy) Panicf(format string, args ...interface{})
- func (p *Proxy) Panicj(j log.JSON)
- func (p *Proxy) Prefix() string
- func (p *Proxy) Print(args ...interface{})
- func (p *Proxy) Printf(format string, args ...interface{})
- func (p *Proxy) Printj(j log.JSON)
- func (p *Proxy) SetHeader(h string)
- func (p *Proxy) SetLevel(l log.Lvl)
- func (p *Proxy) SetOutput(w io.Writer)
- func (p *Proxy) SetPrefix(prefix string)
- func (p *Proxy) Warn(args ...interface{})
- func (p *Proxy) Warnf(format string, args ...interface{})
- func (p *Proxy) Warnj(j log.JSON)
- func (p *Proxy) Write(b []byte) (int, error)
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func SetDefault ¶
func SetDefault(p *Proxy)
Types ¶
type Logger ¶
type Logger interface {
Output() io.Writer
SetOutput(w io.Writer)
Prefix() string
SetPrefix(p string)
Level() log.Lvl
SetLevel(v log.Lvl)
SetHeader(h string)
Print(i ...interface{})
Printf(format string, args ...interface{})
Printj(j log.JSON)
Debug(i ...interface{})
Debugf(format string, args ...interface{})
Debugj(j log.JSON)
Info(i ...interface{})
Infof(format string, args ...interface{})
Infoj(j log.JSON)
Warn(i ...interface{})
Warnf(format string, args ...interface{})
Warnj(j log.JSON)
Error(i ...interface{})
Errorf(format string, args ...interface{})
Errorj(j log.JSON)
Fatal(i ...interface{})
Fatalj(j log.JSON)
Fatalf(format string, args ...interface{})
Panic(i ...interface{})
Panicj(j log.JSON)
Panicf(format string, args ...interface{})
}
Logger defines the logging interface.
type Proxy ¶
type Proxy struct {
// contains filtered or unexported fields
}
Proxy is a proxy type for logrus.Logger
func NewProxyFor ¶
NewProxyFor creates a new Proxy for a particular logger
func NewProxyWithContextFor ¶ added in v0.0.5
NewProxyWithContextFor creates a new Proxy for a particular logger with a particular context.
Click to show internal directories.
Click to hide internal directories.